﻿using System;
using System.Windows;
using System.Windows.Media.Media3D;

namespace TestProject
{
    /// <summary>
    /// 구체 메시 제너레이터
    /// </summary>
    public class SphereMeshGenerator
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 슬라이스 수
        /// </summary>
        private int sliceCount = 32;

        /// <summary>
        /// 스택 수
        /// </summary>
        private int stackCount = 16;

        /// <summary>
        /// 중심점
        /// </summary>
        private Point3D centerPoint = new Point3D();

        /// <summary>
        /// 반경
        /// </summary>
        private double radius = 1;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 슬라이스 수 - SliceCount

        /// <summary>
        /// 슬라이스 수
        /// </summary>
        public int SliceCount
        {
            get
            {
                return this.sliceCount;
            }
            set
            {
                this.sliceCount = value;
            }
        }

        #endregion
        #region 스택 수 - StackCount

        /// <summary>
        /// 스택 수
        /// </summary>
        public int StackCount
        {
            set
            {
                this.stackCount = value;
            }
            get
            {
                return this.stackCount;
            }
        }

        #endregion
        #region 중심점 - CenterPoint

        /// <summary>
        /// 중심점
        /// </summary>
        public Point3D CenterPoint
        {
            get
            {
                return this.centerPoint;
            }
            set
            {
                this.centerPoint = value;
            }
        }

        #endregion
        #region 반경 - Radius

        /// <summary>
        /// 반경
        /// </summary>
        public double Radius
        {
            get
            {
                return this.radius;
            }
            set
            {
                this.radius = value;
            }
        }

        #endregion

        #region 도형 - Geometry

        /// <summary>
        /// 도형
        /// </summary>
        public MeshGeometry3D Geometry
        {
            get
            {
                MeshGeometry3D mesh = new MeshGeometry3D();

                for(int stack = 0; stack <= StackCount; stack++)
                {
                    double phi   = Math.PI / 2 - stack * Math.PI / StackCount;
                    double y     = Radius * Math.Sin(phi);
                    double scale = -Radius * Math.Cos(phi);

                    for(int slice = 0; slice <= SliceCount; slice++)
                    {
                        double theta = slice * 2 * Math.PI / SliceCount;
                        double x     = scale * Math.Sin(theta);
                        double z     = scale * Math.Cos(theta);

                        Vector3D normalVector = new Vector3D(x, y, z);

                        mesh.Normals.Add(normalVector);

                        mesh.Positions.Add(normalVector + CenterPoint);

                        mesh.TextureCoordinates.Add
                        (
                            new Point
                            (
                                (double)slice / SliceCount,
                                (double)stack / StackCount
                            )
                        );
                    }
                }

                for(int stack = 0; stack < StackCount; stack++)
                {
                    int top    = (stack + 0) * (SliceCount + 1);
                    int bottom = (stack + 1) * (SliceCount + 1);

                    for(int slice = 0; slice < SliceCount; slice++)
                    {
                        if(stack != 0)
                        {
                            mesh.TriangleIndices.Add(top    + slice    );
                            mesh.TriangleIndices.Add(bottom + slice    );
                            mesh.TriangleIndices.Add(top    + slice + 1);
                        }

                        if(stack != StackCount - 1)
                        {
                            mesh.TriangleIndices.Add(top    + slice + 1);
                            mesh.TriangleIndices.Add(bottom + slice    );
                            mesh.TriangleIndices.Add(bottom + slice + 1);
                        }
                    }
                }

                return mesh;
            }
        }

        #endregion
    }
}